#HydroR Webinar Script 2
#Develop a model to predict total suspended solids (TSS) from continuous turbidity data
#and assess compliance with water quality criteria at a site in the Little Bear River

#Get the TSS data series from the database
tss1 <- getDataSeries(connectionString="C:/temp/HydroRWebinar/HydroRWebinar.sqlite",
	seriesID=3,
	SQLite=TRUE,
	startDate= "2006-10-19",
	endDate="2007-06-07")

tss2 <- getDataSeries(connectionString="C:/temp/HydroRWebinar/HydroRWebinar.sqlite",
	seriesID=4,
	SQLite=TRUE,
	startDate= "2007-10-04",
	endDate="2009-01-06")

#Get the turbidity data series from the database
turb <- getDataSeries(connectionString="C:/temp/HydroRWebinar/HydroRWebinar.sqlite",
	seriesID=2,
	SQLite=TRUE,
	startDate= "2006-10-18",
	endDate="2009-01-16")

#Combine the two TSS data frames into one data frame (e.g., one time series)
allTSS <- rbind(tss1$DataValues,tss2$DataValues)

#Plot the new dataframe to make sure we got the right result
plot(allTSS$LocalDateTime, allTSS$DataValue, type="p", xlab="Date", ylab="TSS, mg/L")

#Get a turbidity value corresponding to the time of each TSS value by interpolating the continuous
#turbidity values and store the results in a vector
interpolatedTurbidity <- approx(turb$DataValues$LocalDateTime,turb$DataValues$DataValue,allTSS$LocalDateTime,method="linear")$y

#Create a data frame of the data that will be used for the regression
#The structure will be LocalDateTime, TSS, Turbidity
regData <- data.frame(LocalDateTime=allTSS$LocalDateTime,TSS=allTSS$DataValue,Turbidity=interpolatedTurbidity)
head(regData)

#Now plot an x-y scatter plot to look at correlation between turbidity and TSS
plot(regData$Turbidity,regData$TSS,xlab="Turbidity, NTU",ylab="TSS, mg/L",xlim=c(0,60),ylim=c(0,120),xaxs="i",yaxs="i")

#Do a least squares fit of the data to create a linear model
lmTSS <- lm(TSS ~ Turbidity, data=regData)

#Show the summary of the linear model that was created
summary(lmTSS)

#Plot the x-y scatter plot with the regression line
plot(regData$Turbidity,regData$TSS,xlab="Turbidity, NTU",ylab="TSS, mg/L",xlim=c(0,60),ylim=c(0,120),xaxs="i",yaxs="i")
abline(lmTSS, col="red") 

#Create a data frame that converts turbidity to TSS
#The structure will be LocalDateTime, EstimatedTSS
newdata <- data.frame(Turbidity=turb$DataValues$DataValue)
estTSS <- data.frame(LocalDateTime=turb$DataValues$LocalDateTime,EstimatedTSS=predict(lmTSS,newdata))
head(estTSS)

#Create a plot of TSS values estimated from turbidity
plot(estTSS$LocalDateTime,estTSS$EstimatedTSS,type="l",xlab="Date",ylab="Estimated TSS, mg/L")

#Create a TSS concentration duration curve to look at compliance with target WQ criteria
#Rank the values in the estimated TSS time series
rank <- rank(estTSS$EstimatedTSS, ties.method="max")
rank <- max(rank)-rank
#Calculate the percent of time each TSS value is equaled or exceeded
exceedtime <- 100 * (rank / (length(estTSS$EstimatedTSS)+1))
#Plot the TSS concentration duration curve
plot(sort(exceedtime, decreasing=T), sort(estTSS$EstimatedTSS, decreasing=F), 
	type="p", xlab="Percent of time TSS concentration was equaled or exceeded", ylab="TSS, mg/L")
#Add a horizontal line for the water quality target concentration
abline(h=35,col="red")

#Create a new list object for the estimated TSS data series and then write it to the HydroDesktop database
#Create the blank data series using the HydroR createDataSeries function
newDataSeries <- createDataSeries(connectionString="C:/temp/HydroRWebinar/HydroRWebinar.sqlite",SQLite=TRUE)

#Set the site information (use the site from the turb data series)
newDataSeries$Site <- turb$Site

#Set the variable information	(this is a new variable that needs to be created in the database)
newDataSeries$Variable[1,] <- c(NA, "LittleBearRiver:TSS","Solids, total Suspended","Not Applicable",4,
	"Surface Water","Model Simulation Result",1,0,5,3,"Median","Water Quality",-9999,NA)
   
#Set the variable units information (use the units from the tss1 data series - mg/L)
newDataSeries$VariableUnits <- tss1$VariableUnits

#Set the variable time support units inforamtion (use the time support from the turb data series)
newDataSeries$TimeUnits <- turb$TimeUnits

#Set the method information (this is a new method that needs to be created in the database)
newDataSeries$Method[1,] <- c(NA,"Total suspended solids estimated from turbidity using a linear regression model.","Unknown")

#Set the DataSeries information (most of this will get generated automatically when the series is created in the database)
newDataSeries$DataSeries[1, ] <- c(NA, NA, NA, 0, NA, NA, NA, NA, NA, NA, NA, NA, NA, 0, NA, NA)

#Set the source information (use the source information from the turb data series)
newDataSeries$Source <- turb$Source

#Set the quality control level information
newDataSeries$QualityControlLevel[1,] <- c(NA,"2","Derived products","Derived data products.")

#Set the DataValues
newDataSeries$DataValues <- turb$DataValues  #Start with the DataValues data frame from the turbidity dataset
newDataSeries$DataValues$ValueID <- NA  #Get rid of the ValueIDs
newDataSeries$DataValues$SeriesID <- NA #Get rid of the SeriesIDs
newDataSeries$DataValues$DataValue <- estTSS$EstimatedTSS #Replace the turbidity data values with estimated TSS data values

#Now save the data series to the database using the HydroR saveDataSeries function
saveDataSeries(connectionString="C:/temp/HydroRWebinar/HydroRWebinar.sqlite",
	newSeries=newDataSeries,
	SQLite=TRUE,
	overwrite=FALSE)

#Make sure the new data series is a part of the HydroDesktop theme that we created
getDataWithSQL(connectionString="C:/temp/HydroRWebinar/HydroRWebinar.sqlite",
	"INSERT INTO DataThemes VALUES(1,5)",
	SQLite=TRUE)

